
/* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================*/
#include <ncbi.h>
#include <blast/blast.h>

int LIBCALL
BlastWFContextEncodedAdd(wfcp, start, len, T, process)
	BLAST_WFContextPtr	wfcp;
	size_t		start;
	size_t		len;
	BLAST_Score	T; /* ignored */
	int	process; /* ignored */
{
	BLAST_Filter	ff[2];

	ff[0].func = BlastSelfScoreWordFltr;
	ff[0].id = 1;
	ff[0].data = wfcp;
	ff[0].next = &ff[1];

	ff[1].func = BlastWFContextAddEncodedFltr;
	ff[1].id = 2;
	ff[1].data = wfcp;
	ff[1].next = NULL;

	return BlastWFContextEncodedFltrApply(ff, wfcp, start, len, BLAST_SCORE_MAX, FALSE);
}

int LIBCALL
BlastWFContextEncodedFltrApply(ffp, wfcp, start, len, T, slide)
	BLAST_FilterPtr	ffp;
	BLAST_WFContextPtr	wfcp;
	/* The interval over which to apply the filter */
	size_t	start,	/* starting offset */
			len;	/* segment length */
	BLAST_Score	T;
	int	slide;
{
	register BLAST_LetterPtr	str;
	register size_t	off, qlen;
	register int	rc;
	register BLAST_WFContextWordFltrFnPtr	func;

	if (wfcp == NULL || wfcp->query == NULL || wfcp->sbp == NULL)
		return blast_errno = BLAST_ERR_INVAL;

	if (ffp == NULL || (func = ffp->func) == NULL)
		return blast_errno = BLAST_ERR_INVAL;

	str = wfcp->query->str + start;
	qlen = wfcp->query->len;

	if (start >= qlen || start + len > qlen)
		return blast_errno = BLAST_ERR_DOMAIN;

	if (wfcp->W > len)
		return 0;

	for (off = start + wfcp->enclet; off <= start + len; ++off)
		if ((rc = func(ffp, off, str++, wfcp->enclet, wfcp->sbp, T, slide)) != 0)
			return rc;

	return 0;
}

int
BlastWFContextAddEncodedFltr(BLAST_FilterPtr ffp, size_t off, register BLAST_LetterPtr str, int W, BLAST_Score T, int slide)
{
	BLAST_WFContextPtr	wfcp;
	BLAST_WordFinderPtr	wfp;
	DFAPtr	dp;
	DFA_PatID	pi;
	unsigned	context;

	wfcp = ffp->data;
	wfp = wfcp->wfp;
	if (wfp->state != BLAST_WFSTATE_OPEN)
		return blast_errno = BLAST_ERR_WFSTATE;

	dp = wfp->dp;
	++wfcp->stats.neighborhood_words;
	context = wfcp->context;

	if (wfp->lpb == 1) {
		switch (wfp->context_max) {
		case 1:
			pi.u = off;
#if BLAST_LETTER_SIZE == 1
			if (dfa_add(dp, (unsigned char PNTR)str, W, pi) != NULL)
#else
			if (dfa_iadd(dp, (int PNTR)str, W, pi) != NULL)
#endif
				return BLAST_ERR_NONE;
			return (int)(blast_errno = BLAST_ERR_DFA);
			/*NOTREACHED*/
		default:
			pi.u = ((unsigned long)off << BLAST_WORDFINDER_CONTEXT_SHIFT) | context;
#if BLAST_LETTER_SIZE == 1
			if (dfa_add(dp, (unsigned char PNTR)str, W, pi) != NULL)
#else
			if (dfa_iadd(dp, (int PNTR)str, W, pi) != NULL)
#endif
				return BLAST_ERR_NONE;
			return (int)(blast_errno = BLAST_ERR_DFA);
			/*NOTREACHED*/
		}
	}

	{
		BLAST_Letter	el[(BLAST_WORDSIZE_MAX+1)/2];
		BLAST_LetterPtr	elp, strf;
		BLAST_Letter	ch;
		int	i, bpl, lpb;

		if (wfp->type != BLAST_WFTYPE_2)
			return blast_errno = BLAST_ERR_WFTYPE;

		lpb = wfp->lpb;
		bpl = wfp->ap->bpl;
		elp = el;
		strf = str + wfcp->enclet;
		while (str < strf) {
			ch = *str++;
			for (i = 1; i < lpb; ++i) {
				ch <<= bpl;
				ch |= *str++;
			}
			*elp++ = ch;
		}
		pi.u = ((unsigned long)off << BLAST_WORDFINDER_CONTEXT_SHIFT) | context;
#if BLAST_LETTER_SIZE == 1
		if (dfa_add(dp, (unsigned char PNTR)el, wfcp->enclen, pi) != NULL)
#else
		if (dfa_iadd(dp, (int PNTR)el, wfcp->enclen, pi) != NULL)
#endif
			return BLAST_ERR_NONE;
		return (int)(blast_errno = BLAST_ERR_DFA);
	}
	/*NOTREACHED*/
}
