/* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================*/
#include <ncbi.h>
#include <blast/blast.h>

BLAST_FilterPtr
BlastFilterNew(func, id, data)
	int	(*func)(), id;
	Nlm_VoidPtr	data;
{
	BLAST_FilterPtr	fp;

	fp = (BLAST_FilterPtr)BlastMalloc(sizeof(*fp));
	if (fp == NULL)
		return NULL;
	fp->next = NULL;
	fp->func = (int (*)PROTO((struct _blast_filter PNTR)))func;
	fp->id = id;
	fp->data = data;
	return fp;
}

BLAST_Error
BlastFilterDestruct(fpp)
	BLAST_FilterPtr PNTR	fpp;
{
	BLAST_FilterPtr	fp, fp2;

	if (fpp == NULL)
		return blast_errno = BLAST_ERR_INVAL;

	for (fp = *fpp; fp != NULL; fp = fp2) {
		fp2 = fp->next;
		BlastFree(fp);
	}
	*fpp = NULL;
	return BLAST_ERR_NONE;
}

BLAST_FilterPtr
BlastFilterFindByID(fp, id)
	BLAST_FilterPtr	fp;
	int		id;
{
	while (fp != NULL) {
		if (fp->id == id)
			return fp;
		fp = fp->next;
	}
	return NULL;
}

BLAST_FilterPtr
BlastFilterStack(fpp, func, id, data)
	BLAST_FilterPtr PNTR	fpp;
	int		(*func)();
	int		id;
	Nlm_VoidPtr	data;
{
	BLAST_FilterPtr	fp, tail;

	if (fpp == NULL) {
		blast_errno = BLAST_ERR_INVAL;
		return NULL;
	}

	fp = BlastFilterNew(func, id, data);
	if (fp == NULL)
		return NULL;

	if (*fpp == NULL)
		return *fpp = fp;

	for (tail = *fpp; tail->next != NULL; tail = tail->next)
		;
	tail->next = fp;

	return fp;
}

BLAST_Error
BlastFilterPush(fpp, func, id, data)
	BLAST_FilterPtr PNTR	fpp;
	int		(*func)();
	int		id;
	Nlm_VoidPtr	data;
{
	BLAST_FilterPtr	fp;

	if (fpp == NULL)
		return blast_errno = BLAST_ERR_INVAL;

	if (*fpp == NULL) {
		if (BlastFilterStack(fpp, func, id, data) != NULL)
			return BLAST_ERR_NONE;
		return blast_errno;
	}

	fp = BlastFilterNew(func, id, data);
	if (fp == NULL)
		return blast_errno;
	
	fp->next = *fpp;
	*fpp = fp;
	return BLAST_ERR_NONE;
}


BLAST_Error
BlastFilterPop(fpp)
	BLAST_FilterPtr PNTR	fpp;
{
	BLAST_FilterPtr	fp;

	if (fpp == NULL)
		return blast_errno = BLAST_ERR_INVAL;
	if (*fpp == NULL)
		return blast_errno = BLAST_ERR_INVAL;
	fp = *fpp;
	*fpp = fp->next;
	BlastFree(fp);
	return BLAST_ERR_NONE;
}
