/* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================*/
#ifndef __BLAST_SEARCH__
#define __BLAST_SEARCH__
#ifdef __cplusplus
extern "C" {
#endif

typedef int (LIBCALLBACK *BLAST_HSPFltrFnPtr) PROTO((BLAST_FilterPtr,BLAST_WFContextPtr,BLAST_HSPPtr));

typedef int (LIBCALLBACK *BLAST_ExtendNtOrigFnPtr) PROTO((BLAST_FilterPtr,BLAST_WFContextPtr,size_t,size_t,int,int));


#define RES_L2B4_L0(x) ((x)>>4) /* x must be unsigned to avoid sign extension */
#define RES_L2B4_L1(x) ((x) & 0x0f)

#define RES_L4B2_L0(x) ((x)>>6) /* x must be unsigned to avoid sign extension */
#define RES_L4B2_L1(x) (((x)>>4) & 0x03)
#define RES_L4B2_L2(x) (((x)>>2) & 0x03)
#define RES_L4B2_L3(x) ((x) & 0x03)

/*
    Function naming convention:

    BlastWordFinder_F#L#(B#)

    F# -- no. of contexts
    L# -- no. of letters per byte
    B# -- no. of bits per packed letter (used only when L# > 1)


    BlastWordExtendFltr_L#(B#)

    L# -- no. of letters per byte
    B# -- no. of bits per packed letter (used only when L# > 1)

	"Stats" versions tally statistics (which adds unnecessary overhead
	in some applications)
*/
int (LIBCALLBACK *BlastWordFinderSelect PROTO((BLAST_WordFinderPtr))) ();
int (LIBCALLBACK *BlastWordFinderSelectStats PROTO((BLAST_WordFinderPtr))) ();

int LIBCALL BlastWordFinder_F1L1 PROTO((BLAST_WordFinderPtr wfp));
int LIBCALL BlastWordFinder_F1L1Stats PROTO((BLAST_WordFinderPtr wfp));
int LIBCALL BlastWordFinder_FnL1 PROTO((BLAST_WordFinderPtr wfp));
int LIBCALL BlastWordFinder_FnL1Stats PROTO((BLAST_WordFinderPtr wfp));
int LIBCALL BlastWordFinder_FnL2B4 PROTO((BLAST_WordFinderPtr wfp));
int LIBCALL BlastWordFinder_FnL2B4Stats PROTO((BLAST_WordFinderPtr wfp));
int LIBCALL BlastWordFinder_FnL4B2 PROTO((BLAST_WordFinderPtr wfp));
int LIBCALL BlastWordFinder_FnL4B2Stats PROTO((BLAST_WordFinderPtr wfp));

int LIBCALL BlastHSPExtendNoXFltr_L1 PROTO((BLAST_FilterPtr ffp, BLAST_WFContextPtr wfcp, BLAST_HSPPtr hp));
int LIBCALL BlastHSPExtendNoXFltr_L1Stats PROTO((BLAST_FilterPtr ffp, BLAST_WFContextPtr wfcp, BLAST_HSPPtr hp));

BLAST_Error LIBCALL	BlastWordFinderSearch PROTO((BLAST_WordFinderPtr,BLAST_StrPtr));
int LIBCALL search_nt_orig PROTO((BLAST_WordFinderPtr wfp));
int LIBCALL search_nt_origStats PROTO((BLAST_WordFinderPtr wfp));

#ifdef __cplusplus
}
#endif
#endif /* !__BLAST_SEARCH__ */
