/* ===========================================================================
*
*                            PUBLIC DOMAIN NOTICE
*               National Center for Biotechnology Information
*
*  This software/database is a "United States Government Work" under the
*  terms of the United States Copyright Act.  It was written as part of
*  the author's official duties as a United States Government employee and
*  thus cannot be copyrighted.  This software/database is freely available
*  to the public for use. The National Library of Medicine and the U.S.
*  Government have not placed any restriction on its use or reproduction.
*
*  Although all reasonable efforts have been taken to ensure the accuracy
*  and reliability of the software and data, the NLM and the U.S.
*  Government do not and cannot warrant the performance or results that
*  may be obtained by using this software or data. The NLM and the U.S.
*  Government disclaim all warranties, express or implied, including
*  warranties of performance, merchantability or fitness for any particular
*  purpose.
*
*  Please cite the author in any work or product based on this material.
*
* ===========================================================================*/

#include <ncbi.h>
#include <blast/blast.h>

BLAST_HSPPtr LIBCALL
BlastSegSetMax(hlp, sbp, kbp, sp, start, len, cutoff)
	BLAST_HitListPtr	hlp;
	BLAST_ScoreBlkPtr	sbp;
	BLAST_KarlinBlkPtr	kbp;
	BLAST_StrPtr		sp;
	BLAST_Coord			start, len;
	BLAST_Score			cutoff;
{
	BLAST_LetterPtr	q, qmax, qbeg, qend;
	BLAST_Score	sum, score;
	BLAST_HSP	hsp;
	BLAST_HSPPtr	hp = NULL;

	if (hlp == NULL || sbp == NULL || sp == NULL) {
		blast_errno = BLAST_ERR_INVAL;
		return NULL;
	}
	if (sbp->a1 != sp->ap) {
		blast_errno = BLAST_ERR_ALPHACONFLICT;
		return NULL;
	}
	if (start >= sp->len || len < 0 || start + len > sp->len) {
		blast_errno = BLAST_ERR_DOMAIN;
		return NULL;
	}
	if (sp->lpb != 1) {
		blast_errno = BLAST_ERR_ENCODING;
		return NULL;
	}

	score = sum = 0;
	q = sp->str + start;
	for (qbeg = q, qmax = q + len; q < qmax; ++q) {
		if ((sum += sbp->maxcost[*q]) <= 0) {
			if (score > 0) {
Check:
				q = qend;
				if (score >= cutoff) {
					hsp.hlp = NULL;
					hsp.next = NULL;
					hsp.vsp = NULL;
					hsp.kbp = kbp;
					hsp.sbp = sbp;
					hsp.q_seg.sp = sp;
					hsp.s_seg.sp = sp; /* really should be NULL */
					hsp.q_seg.frame = sp->frame;
					hsp.s_seg.frame = 0;
					hsp.score = score;
					hsp.len = qend - qbeg + 1;
					hsp.q_seg.offset = hsp.s_seg.offset = qbeg - sp->str;
					hp = BlastHSPSave(hlp, &hsp);
				}
			}
			qbeg = qend = q;
			score = sum = 0;
		}
		else
			if (sum > score) {
				score = sum;
				qend = q;
			}
	} /* for (;;) */
	if (score > 0)
		goto Check;

	return hp;
}
